/*****************************************************************/
/*          S A S   A U T O C A L L   L I B R A R Y              */
/*                                                               */
/*   NAME : CIF                                                  */
/*   TITLE: Estimate Cumulative Incidence Function and           */
/*          Compare Cumulative Incidence Functions across Samples*/
/* PRODUCT: STAT                                                 */
/*  SYSTEM: ALL                                                  */
/*    KEYS: survival data, failure probability, competing risks  */
/*   PROCS:                                                      */
/*    DATA:                                                      */
/* SUPPORT: Guixian Lin                UPDATE:  Mar2012          */
/*     REF: Klein and Moeschberger    (2003)                     */
/*          Marubini and Valsecchi    (1995)                     */
/*          Gray                      (1988)                     */
/*    MISC: The %CIF macro requires Base SAS and SAS/IML.        */
/*          SAS/GRAPH is also required in SAS 9.2                */
/*    (Zhiwei Wang:)                                             */
/*    Added N at risk,                                           */
/*    Some documentation removed.                                */
/*    (Zhen-Huan Hu:)                                            */
/*    Added arcsine-square root transformation,                  */
/*    Some code cleaning.                                        */
/*    For complete documentation, visit SAS Knowledge base       */ 
/*    Sample 45997.                                              */
/*****************************************************************/

%macro cif(data =, out = cifestimate, time = time, status = status, event = 1, censored = 0, group =, strata =, alpha = .05, rho = 0, conftype = 1, se = 1, title =, title2 =, options =);
  %* data:		Name of the input data set;
  %* out = cifestimate: Name of the output data set that contains the cumulative incidence estimatte;
  %* time = time:       Time variable;
  %* status = status:   Numeric variable whose values indicate  whether an observation corresponds to the event of interest, the competing event, or is censored;
  %* event = 1:         Values of status= variable that correspond to the event interest. Multiple values are separated by blanks;
  %* censored = 0:      Values of status= variable that correspond to censored observations. Multiple values are separated blanks;
  %* group:		Variable that defines the groups for comparison;
  %* strata:		Variable that defines the  strata for stratified test;
  %* alpha = .05:       Complement of the confidence level for pointwise confidence limits;
  %* rho = 0:           Power of the weight function in the test;
  %* conftype = 1:	Method to compute confidence limits. 1 = log-log transformation, 2 = arcsine-square root transformation, 3 = logit transformation;
  %* se = 1:            Method to compute the standard error of cumulative incidence functions. 1 = delta method, 2 = counting process;
  %* title:		First title for the cumulative incidence function plot (without quote);
  %* title2:		Second title for the cumulative incidence function plot (without quote);
  %* options:		Multiple keywords are separated by blanks NOPLOT suppresses plotting the cumulative incidence curves;
  %*			PLOTCL plots the pointwise confidence limits for cumulative incidence curves;
  %*			NOCIFEST suppresses the CIF estimate table;
  %*			NOTEST suppresses the test for differences among cumulative incidence functions;

  %local confidence savenote noplotflag plotclflag notestflag nocifflag nev s11 inconsistentgrpnumflag;
  %let savenote = %sysfunc(getoption(notes));
  options nonotes;

  %* Check TIME = and STATUS = variables okay;
  %let time = %lowcase(&time);
  %let status = %lowcase(&status);
  %let group = %lowcase(&group);
  %let strata = %lowcase(&strata);

  %let timevar = 0;
  %let statusvar = 0;
  %let groupvar = 0;
  %let stratavar = 0;
  %let nocifflag = 0;
  %let inconsistentgrpnumflag = 0;

  %let dsid = %sysfunc(open(&data));
  %do i = 1 %to %sysfunc(attrn(&dsid, nvars));
    %let var = %lowcase(%sysfunc(varname(&dsid, &i)));
    %if &var = &time %then %do;
      %if %sysfunc(vartype(&dsid, &i)) = N %then %let timevar = 1;
      %else %if %sysfunc(vartype(&dsid, &i)) = C %then %let timevar = 2;
    %end;
    %else %if &var = &status %then %do;
      %if %sysfunc(vartype(&dsid, &i)) = N %then %let statusvar = 1;
      %else %if %sysfunc(vartype(&dsid, &i)) = C %then %let statusvar = 2;
    %end;
    %else %if &var = &group %then %let groupvar = 1;
    %else %if &var = &strata %then %let stratavar = 1;
  %end;
  %let rc = %sysfunc(close(&dsid));

  %if &timevar = 0 %then %do;
    %put ERROR: Variable &time not found in the input data set.;
    %goto exit;
  %end;
  %else %if &timevar = 2 %then %do;
    %put ERROR: Variable &time is not a numeric variable.;
    %goto exit;
  %end;
  %if &statusvar = 0 %then %do;
    %put ERROR: Variable &status not found in the input data set.;
    %goto exit;
  %end;
  %else %if &statusvar = 2 %then %do;
    %put ERROR: Variable &status is not a numeric variable.;
    %goto exit;
  %end;
  %if &group ne and &groupvar = 0 %then %do;
    %put ERROR: Variable &group not found in the input data set.;
    %goto exit;
  %end;
  %if &strata ne and &stratavar = 0 %then %do;
    %put ERROR: Variable &strata not found in the input data set.;
    %goto exit;
  %end;

  /* Check OPTIONS = */
  %let noplotflag = %sysfunc(prxmatch(%str(/noplot/i), &options));
  %let plotclflag = %sysfunc(prxmatch(%str(/plotcl/i), &options));
  %let notestflag = %sysfunc(prxmatch(%str(/notest/i), &options));
  %let nocifestflag = %sysfunc(prxmatch(%str(/nocifest/i), &options));

  /* Confidence level */
  %if &alpha eq %then %let alpha = .05;
  %let confidence = %sysevalf(100 * (1 - &alpha), integer);

  /* # of EVENT = and COMPETE = values */
  %let ne = %sysfunc(countw(&event));
  %let ncens = %sysfunc(countw(&censored));

  %if &data eq %then %do;
    %put ERROR: The DATA = parameter is not specified.;
    %goto exit;
  %end;
  %else %do;
    data _null_;
      set &data;
      array event[&ne] _temporary_(&event);
      array censored[&ncens] _temporary_(&censored);;
      zevent = 0;
      zcensored = 0;
      do i = 1 to &ne;
        if &status = event[i] then do;
          zevent = 1;
          leave;
        end;
      end;

      do i = 1 to &ncens;
        if &status = censored[i] then do;
          zcensored = 1;
          leave;
        end;
      end;

      if zevent = 1 and zcensored = 1 then do;
        call symput('s11', strip(put(&status, best8.)));
        stop;
      end;
      if zevent = 1 then do;
        call symput('nev', put(1, 1.));
      end;
    run;

    %if &nev eq %then %do;
      %put ERROR: The total number of EVENT is 0.;
      %goto exit;
    %end;
    %else %if &s11 ne %then %do;
      %put ERROR: &status = %sysfunc(trim(&s11)) is both an EVENT = and CENSORED = value.;
      %goto exit;
    %end;

    data _null_;
      set &data;
      length l $ 60;
      call vname(&time, l);
      call symputx('tname', l, 'L');
      call label(&time, l);
      call symputx('tlabel', l, 'L');
      %if &group ne %then %do;
        call vname(&group, l);
        call symputx('gname', l, 'L');
        call label(&group, l);
        call symputx('glabel',l , 'L');
        call symputx('grpfmt', VFORMAT(&group));
      %end;
      %else %do;
        call symputx('glabel',"Group" , 'L');
        call symputx('gname', "Group", 'L');;
      %end;

      %if &strata ne %then %do;
        call vname(&strata, l);
        call symputx('stratname', l, 'L');
        call label(&strata, l);
        call symputx('stratlabel',l , 'L');
        call symputx('stratfmt', VFORMAT(&strata));
      %end;
      %else %do;
        call symputx('stratlabel',"Strata", 'L');
        call symputx('stratname', "Strata", 'L');;
      %end;
    run;
  %end;

  /* Create a data set excluding those missing observations */
  data &data.init(drop = &group);
    set &data;
    %if &group ne %then %do;
      f&group = put(&group, &grpfmt);
      if cmiss(&time, &status, f&group) > 0 then delete;
    %end;
    %if &strata ne %then %do;
      f&strata = put(&strata, &stratfmt);
      if cmiss(&time, &status, f&strata) > 0 then delete;
    %end;
  run;

  proc iml;
    use &data.init;
    read all var{&time} into time;
    read all var{&status} into status;
    nobs = nrow(time);
    %if %bquote(&group) eq %then %do; group = j(nobs, 1, 1); %end;
    %else %do; read all var{f&group} into group; %end;
    %if %bquote(&strata) eq %then %do; strat = j(nobs, 1, 1); %end;
    %else %do; read all var{f&strata} into strat; %end;
    close &data.init;

    %* Subroutine to calculate cumulative incidence and variance;
    start estimate_cif(y, causecode) global(time, nriskv, cif, vcif1, vcif2, n2);
    %* y is the failure times (sorted in increasing order);
    %* ic is the indicator for censoring (= 0 if censored, = 1 if failed from any cause);
    %* icc is the indicator for failure from the cause of interest (= 0 if failed from cause of interest, = 1 otherwise);
    %* n is the number of observations (length of y);
    %* n2 = n + 2: the number of unique time points from cause 1 with 2 extra points (0 and largest time point within this group);
    %* ut is the unique failure times from the cause of interest;
    %* cif is the estimated cumulative incidence function;
    %* vcif1 and vcif2 are the estimated variance of CIF from Marubini method and Gray method respectively;

    surv_km = 1;
    cif_pos = 1;

    v11 = 0; v21 = 0;
    v12 = 0; v22 = 0;
    v13 = 0; v23 = 0;

    n = nrow(y);
    n2 = ncol(unique(y[loc(causecode = 1)])) + 2;
    ut = unique(y); %* Unique failure times;
    n_ut = ncol(ut); %* Number of unique failure time;

    cif = j(n2, 1, 0);
    vcif1 = j(n2, 1, 0);
    vcif2 = j(n2, 1, 0);
    nriskv = j(n2, 1, 0);

    nrisk = n;
    nriskv[1] = nrisk;

    do i = 1 to n_ut; %* Do the loop for each unique failure time point;
      %* Compute the number of events of interest and competing events
      %* that happened at each unique failure time point;
      nevent = sum(causecode[loc(y = ut[i])] = 1);
      ncompetevent = sum(causecode[loc(y = ut[i])] = 2);
      nall = nevent + ncompetevent;

      %* Compute survival function and cumulative incidence function;
      if nall ^= 0 then do; %* Overall survival changes whenever a failure occurs;
	if i = 1 then nrisk = n;
        else if i > 1 then nrisk = n - min(loc(y = ut[i])) + 1;
        surv_km_n = surv_km * (nrisk - nall) / nrisk;
        if nevent > 0 then do;
          cif_pos = cif_pos + 1;
          cif[cif_pos] = cif[cif_pos - 1] + surv_km * nevent / nrisk;
          nriskv[cif_pos] = nrisk;
        end;
              
        %* Variance estimator using delta method - (10.12) in Marubini and Valsecchi (1995);
        %* Analysing Survival Data from Clinical Trials and Observational Studies;
        t1 = 0;
        if (nrisk - nall > 0) then t1 = nall / nrisk / (nrisk - nall);
        t2 = surv_km * nevent / nrisk / nrisk;
        t3 = surv_km * (1 - nevent / nrisk) * t2;
        t4 = cif[cif_pos] * cif[cif_pos];
        v11 = v11 + t1;
        v12 = v12 - 2 * (cif[cif_pos] * t1 + t2);
        v13 = v13 + t4 * t1 + 2 * cif[cif_pos] * t2 + t3;
        vcif1[cif_pos] = t4 * v11 + cif[cif_pos] * v12 + v13;

        %* Variance estimator using counting process - Gray (1988);
        if (ncompetevent > 0 & surv_km_n > 0) then do;
          vt1 = 1;
          if (ncompetevent > 1) then vt1 = 1 - (ncompetevent - 1) / (nrisk - 1);
          vt2 = surv_km * surv_km * vt1 * ncompetevent/(nrisk * nrisk);
          vt3 = 1 / surv_km_n;
          vt4 = cif[cif_pos] / surv_km_n;
          v21 = v21 + vt2 * vt4 * vt4;
          v22 = v22 + vt2 * vt3 * vt4;
          v23 = v23 + vt2 * vt3 * vt3;
        end;
        if (nevent > 0) then do;
          vt1 = 1;
          if (nevent > 1) then vt1 = 1 - (nevent - 1) / (nrisk - 1);
          vt2 = surv_km * surv_km * vt1 * nevent / (nrisk * nrisk);
          if surv_km_n > 0 then vt3 = 1 / surv_km_n; else vt3 = 0;
          vt4 = 1 + vt3 * cif[cif_pos];
          v21 = v21 + vt2 * vt4 * vt4;
          v22 = v22 + vt2 * vt3 * vt4;
          v23 = v23 + vt2 * vt3 * vt3;
          vcif2[cif_pos] = v21 - 2 * cif[cif_pos] * v22 + cif[cif_pos] * cif[cif_pos] * v23;
        end;

        surv_km = surv_km_n;
      end;
    end; %* End of do loop;

    if cif_pos < n2 then do;
      %* Deal with the largest time point with the group;
      cif_pos = cif_pos + 1;
      time[cif_pos] = ut[n_ut];
      cif[cif_pos] = cif[cif_pos - 1];
      vcif1[cif_pos] = vcif1[cif_pos - 1];
      vcif2[cif_pos] = vcif2[cif_pos - 1];
      nriskv[cif_pos] = nrisk;
    end;
    if (sum(vcif1 < 0) > 0) then vcif1[loc(vcif1 < 0)] = 0;
    if (sum(vcif2 < 0) > 0) then vcif2[loc(vcif2 < 0)] = 0;

    vcif1 = sqrt(vcif1);
    vcif2 = sqrt(vcif2);

    finish estimate_cif;

    %* Subroutine for calculating k-sample test statitsics and variance
    %* for competing risk within each stratum (Gray, 1988);
    start test_ksample(y, causecode, ig, rho) global(score, vcov);
    %* y is the failure times (sorted in ascending order)
    %* causecode is cause indicator
    %* (= 0 if censored, = 1 if failed from the cause of interest, = 2 if failed from some other causes.)
    %* ig denotes group membershi
    %* rho is the power of the weight function in the test statistic
    %* ng denotes the number of groups, ng1 = ng - 1;
    
    ng = ncol(unique(ig));
    %* Risk set size in each at the current failure time (= sample size at 0);
    nrisk = j(ng, 1, 0);

    do i = 1 to ng;
      nrisk[i] = nrisk[i] + sum(ig = i);
    end;

    ng1 = ng - 1;
    ng2 = ng * (ng - 1) / 2;
    score = j(ng1 ,1 , 0);
    vcov = j(ng2, 1, 0);

    l= 0;
    cifgleft= j(ng,1,0);
    cifg=  j(ng,1,0);
    cif0left= 0;
    cif0= 0;

    vtvec=  j(ng,1,0);
    skmgleft= j(ng,1,1);
    skmg= j(ng,1,1);

    vtmatrix= j(ng1,ng,0);
    c= j(ng,ng,0);

    ll= 1;

    ut=unique(y)`;
    n_ut=nrow(ut);

    do i=1 to n_ut;
      d= j(3,ng,0);
      yloc=loc(y=ut[i]);
      do ii= yloc[1] to yloc[ncol(yloc)];
        k= causecode[ii]+ 1;   * causes code:0, 1, 2;
        d[k,ig[ii]]= d[k,ig[ii]] + 1;
      end;
      nd1= d[2,+];
      nd2= d[3,+];

      if (nd1^= 0 | nd2^= 0) then do;
        *hdot is sum(hhat), Rdot is sum(R), skmgleft is Shat, nrisk is Y, cifgleft is CIF;
        hdot= 0; Rdot= 0;
        do g= 1 to ng;
          if (nrisk[g]> 0) then do;
            td= d[2,g] + d[3,g];
            skmg[g]= skmgleft[g]* (nrisk[g]- td)/ nrisk[g];
            cifg[g]= cifgleft[g]+ skmgleft[g]* d[2,g]/ nrisk[g];
            hdot= hdot+ nrisk[g]/ skmgleft[g];
            Rdot= Rdot+ nrisk[g]* (1 - cifgleft[g])/ skmgleft[g];
          end;
        end;

        cif0= cif0left+ nd1/hdot;
        grho= (1- cif0left)**rho;

        * score statistic;
        do g=1 to ng1;
          if (nrisk[g]> 0) then score[g]= score[g]+ grho* (d[2,g]- nd1* nrisk[g]* (1- cifgleft[g])/ skmgleft[g]/ Rdot);
        end;

        a= j(ng, ng,0);
        do g= 1 to ng;
          if (nrisk[g]> 0) then do;
            t1= nrisk[g]/ skmgleft[g];
            a[g,g]= grho* t1* (1- t1/ hdot);
            c[g,g]= c[g,g]+ a[g,g]* nd1/hdot/(1- cif0left);
            k= g + 1;
            *if (k<= ng) then do j= k to ng;
              do j= g+1 to ng;
                if (nrisk[j]> 0) then do;
                  a[g,j]= - grho* t1* nrisk[j]/ skmgleft[j]/ hdot; a[j,g]= a[g,j];
                  c[g,j]= c[g,j]+ a[g,j]* nd1/ hdot/ (1 - cif0left); c[j,g]= c[g,j];
                end;
              end;
            end;
          end;


          * variance estimators;
          if nd1> 0 then do k=1 to ng; 
            if (nrisk[k]> 0) then do;
              if skmg[k]> 0 then vt1= 1- (1- cif0)/ skmg[k]; else vt1= 1;
              if nd1> 1 then vt2= 1- (nd1- 1)/ (hdot* skmgleft[k]- 1); else vt2= 1;
              vt3= vt2* skmgleft[k]* nd1/ (hdot* nrisk[k]);
              vtvec[k]= vtvec[k]+ vt1* vt1* vt3;
              do g= 1 to ng1;
                vt4= a[g,k]- vt1* c[g,k];
                vtmatrix[g,k]= vtmatrix[g,k]+ vt4* vt1* vt3;
                do j=1 to g;
                  l= g* (g- 1)/ 2+ j;
                  vt5= a[j,k]- vt1* c[j,k];
                  vcov[l]= vcov[l]+ vt3* vt4* vt5;
                end;
              end;
            end;
          end;

          if (nd2^= 0) then do k=1 to ng;
            if (skmg[k]> 0 & d[3,k]> 0) then do;
              vt1= (1- cif0)/ skmg[k];
              vt2= 1;
              if (d[3,k]> 1) then vt2= 1- (d[3,k] - 1)/ (nrisk[k]- 1);
              vt3= vt2* ((skmgleft[k]** 2)* d[3,k])/ (nrisk[k]** 2);
              vtvec[k]= vtvec[k]+ vt1* vt1* vt3;
              do g=1 to ng1;
                vt4= vt1* c[g,k];
                vtmatrix[g,k]= vtmatrix[g,k]- vt4* vt1* vt3;
                do j=1 to g;
                  l= g* (g- 1)/ 2+ j;
                  vt5= vt1* c[j,k];
                  vcov[l]= vcov[l]+ vt3* vt4* vt5;
                end;
              end;
            end;
          end; /*if (nd2^= 0)*/
        end; /*if (nd1^= 0 | nd2^= 0)*/

        *update the risk sets and the index of the next failure time;

        do ii= yloc[1] to yloc[ncol(yloc)];
          nrisk[ig[ii]]= nrisk[ig[ii]] - 1;
        end;
        cif0left= cif0;
        cifgleft= cifg;
        skmgleft= skmg;
      end;  /* end do i=*/ /* OK HERE */

      pos= 0;
      do g= 1 to ng1;
        do j= 1 to g;
          pos= pos+ 1;
          do k= 1 to ng;
            vcov[pos]= vcov[pos]+ c[g,k]* c[j,k]* vtvec[k];
            vcov[pos]= vcov[pos]+ c[g,k]* vtmatrix[j,k];
            vcov[pos]= vcov[pos]+ c[j,k]* vtmatrix[g,k];
          end;
        end;
      end;

      finish test_ksample;

      %* Subroutines end and calculation starts here;
      %* Status need to be numerical;
      x = time || status;
      call sortndx(sidx, x, {1 2});
      x = x[sidx, ];

      group = group[sidx];
      strat = strat[sidx];
      ugg = unique(group);
      ng = ncol(ugg);
      call symput('ng', left(char(ng)));

      %* censind = 0 if censored, = 1 if failed;
      censlist = {&censored}`;
      eventlist = {&event}`;
      censind = j(nrow(x), 1, 0);
      eventind = j(nrow(x), 1, 0);
      do i = 1 to &ncens;
        censind = censind | (x[, 2] = censlist[i]);
      end;
      do i= 1 to &ne;
        eventind = eventind | (x[, 2] = eventlist[i]);
      end;

      causecode = 2 * (^censind) - eventind;

      %* Prepare the data;
      x = x || censind || causecode;
      if ng > 1 then do;
        ng1 = ng - 1;
        ng2 = ng * ng1 / 2;
        vv = j(ng1, ng1, 0); %* The final covariance matrix;
        vcov_st = j(ng2, 1, 0);
	vcov = vcov_st; %* vcov_st for each stratum, vs add v across strata;
        score_st = j(ng1, 1, 0);
       	score = score_st; %* s is test statiatics by stratum, ss add s across strata;
      end;

      ust = unique(strat);
      nstrata = ncol(ust);
      call symput('nstrata', left(char(nstrata)));

      /* Calculate CIF and variance */
      if (&nocifflag = 0) then do;
        do k = 1 to nstrata;
          do g = 1 to ng;
            %* Subset of data for group g and stratum k;
            grpstra = (group = ugg[g]) & (strat = ust[k]) & (x[, 4] = 1);

            if (sum(grpstra) > 0) then do;
              xsub = x[loc((group = ugg[g]) & (strat= ust[k])), ];

              run estimate_cif(xsub[, 1], xsub[, 4]);

              time1 = 0 // (unique(xsub[loc(xsub[, 4] = 1), 1]))` // xsub[nrow(xsub), 1];
              tmp = time1 || cif || vcif1 || vcif2; %* Time, CI, variance;

              if (g = 1 & k = 1) then do;
                cifest = tmp;
                nriskm = nriskv;
                grplabel = j(n2, 1, ugg[g]);
                if (nstrata > 1) then stratlabel = j(n2, 1, ust[k]);
              end;
              else do;
                cifest = cifest // tmp;
                nriskm = nriskm // nriskv;
                grplabel = grplabel // j(n2, 1, ugg[g]);
                if (nstrata > 1) then stratlabel = stratlabel // j(n2, 1, ust[k]);
              end;
            end;
          end;
        end;

        /* Create SAS data set for CIF estimate */
        cifloc = loc(cifest[, 2] > 0); %* Location of none-zero CIF estimates;
        ncif = nrow(cifest); %* Number of CIF estimates;
	se = j(ncif, 1, 0); %* Initialize vector for standard errors of CIF estimates;
	tse = j(ncif, 1, 0); %* Initialize vector for standard errors of transformed CIF estimates;
        lowci = j(ncif, 1, 0); %* Initialize vector for lower confidence limits;
        uppci = j(ncif, 1, 0); %* Initialize vector for upper confidence limits;
        z = probit(1 - &alpha / 2.0); %* z = 1.96 when alpha = 0.05, two-tail;

	if (&se = 1) then se[cifloc] = cifest[cifloc, 3]; %* Delta method - Marubini;
	else if (&se = 2) then se[cifloc] = cifest[cifloc, 4]; %* Counting process method - Gray;

	if (&conftype = 1) then do;
	  /* Confidence intervals: log-log tranformation */
	  tse[cifloc] = se[cifloc] / (cifest[cifloc, 2] # log(cifest[cifloc, 2]));
	  lowci[cifloc] = cifest[cifloc, 2] ## exp(-z * tse[cifloc]);
	  uppci[cifloc] = cifest[cifloc, 2] ## exp(z * tse[cifloc]);
	end;
	else if (&conftype = 2) then do;
	  /* Confidence intervals: arcsine-square root transformation */
	  tse[cifloc] = 0.5 * se[cifloc] / sqrt((cifest[cifloc, 2] # (1 - cifest[cifloc, 2])));
	  lowci[cifloc] = sin(0 <> (arsin(sqrt(cifest[cifloc, 2])) - z * tse[cifloc])) ## 2;
	  uppci[cifloc] = sin((3.1416 / 2) >< (arsin(sqrt(cifest[cifloc, 2])) + z * tse[cifloc])) ## 2;
	end;
	else if (&conftype = 3) then do;
	  /* Confidence intervals: logit transformation */
	  tse[cifloc] = se[cifloc] / (cifest[cifloc, 2] # (1 - cifest[cifloc, 2]));
	  lowci[cifloc] = cifest[cifloc, 2] / (cifest[cifloc, 2] + (1 - cifest[cifloc, 2]) # exp(z * tse[cifloc]));
	  uppci[cifloc] = cifest[cifloc, 2] / (cifest[cifloc, 2] + (1 - cifest[cifloc, 2]) # exp(-z * tse[cifloc]));
	end;
        
	cifest = cifest[, (1 : 2)] || se || lowci || uppci || nriskm;
        create &out.plot from cifest[colname = {"&tname" "CIF" "StdErr" "LowerCI" "UpperCI" "Natrisk"}];
        append from cifest;
        close &out.plot;

        create outgrplabel from grplabel[colname = {"&gname"}];
        append from grplabel;
        close outgrplabel;

        if (nstrata > 1) then do;
          create outstratlabel from stratlabel[colname = {"&stratname"}];
          append from stratlabel;
          close outstratlabel;
        end;
      end;

      /* Gray's K-sample test (strata is allowed) */
      if (ng > 1 & (^&notestflag)) then do;
        /* Recode the group member ID from 1 to ng */
        grpcode = j(nrow(x), 1, 0);
        do g = 1 to ng;
          grpcode[loc(group = ugg[g])] = g;
        end;

        do k = 1 to nstrata;
          * Subset of data for stratum k;
          xsub = x[loc(strat = ust[k]), ];
          grpcodesub = grpcode[loc(strat = ust[k])];
          if (ncol(unique(grpcodesub)) = ng) then do;
            run test_ksample(xsub[, 1], xsub[, 4], grpcodesub, &rho);
            score_st = score_st + score;
            vcov_st = vcov_st + vcov;
          end;
          else call symput('inconsistentgrpnumflag', left(char(1)));
        end;

        pos = 0;
        do i = 1 to ng1;
          do j = 1 to i;
            pos = pos + 1;
            vv[i, j] = vcov_st[pos];
            vv[j, i] = vv[i, j];
          end;
        end;

        * Test statistic and p-value;
        test_stat = score_st` * ginv(vv) * score_st;
        df = ng1;
        pval = 1 - probchi(test_stat, df); 
        outtest =  test_stat || df || pval;
        create testresult from outtest[colname = {"test_stat" "df" "pval"}];
        append from outtest;
        close testresult;
      end;

      quit; /*Exit Proc IML*/
    run;

    /*create the output data for the cumulative incidence estimates for the CIF plot */
    %if &nocifflag %then %do; 
      %put WARNING: No CIF is computed when both STRATA and GROUP are specified; 
      %goto exit2; 
    %end;
    data &out.plot;
      merge outgrplabel &out.plot;
      if (0) then set &data(keep=&tname);
    run;

    %if (&nstrata>1) %then %do;
      data &out.plot;
        label &stratname="&stratlabel";
        merge outstratlabel &out.plot;
      run;
    %end;

    data &out(%if %bquote(&group)= %then drop=&gname;);
      set &out.plot;
      %if (&nstrata>1) %then %do;
        by &stratname &gname;
      %end;
      %else %do;
        by &gname;;
      %end;
      if not last.&gname;
    run;

    %if (^&nocifestflag) %then  %do;
      title "Cumulative Incidence Estimates with &Confidence% Confidence Limits";
      proc print data = &out noobs label;
        label LowerCI = "Lower &Confidence% Limits"
        UpperCI = "Upper  &Confidence% Limits";
      run;
    %end;

    /*plot the CIF curves*/

    %if &noplotflag %then %goto exit2;
    %if (%bquote(&title)=) %then
    %if (&plotclflag ) %then
    %let title= Cumulative Incidence Functions with &Confidence% Confidence Intervals;
    %else %let title= Cumulative Incidence Functions;;

    title '&title';
    title2 '&title2';

    %if (&ng>1) %then %let groupopt=group=&gname;
    %else %let groupOpt=;

    %if (&nstrata=1) %then %do;
      proc sgplot data=&out.plot;
        label CIF='Cumulative Incidence';
        *yaxis values=(0 .1 .2 .3 .4 .5 .6 .7 .8 .9 1.0);
        step y=CIF x=&tname / &groupOpt  name="cif";
        %if &plotclflag %then
        band x=&tname lower=LowerCI upper=UpperCI / group=&gname fill transparency=.5
        modelname='cif' legendlabel='';;
      run;
    %end;
    %else %do;
      proc sgpanel data=&out.plot;
        panelby &stratname;
        label CIF='Cumulative Incidence';
        *rowaxis values=(0 .1 .2 .3 .4 .5 .6 .7 .8 .9 1.0);

        step y=CIF x=&tname / &groupOpt  name="cif";
        %if &plotclflag %then
        band x=&tname lower=LowerCI upper=UpperCI / group=&gname fill transparency=.5
        modelname='cif' legendlabel='';;

      run;
    %end;

    %exit2:
    %if (&inconsistentgrpnumflag) %then %do; 
      %put WARNING: The test is not available because the &group number is not the same across the strata;
      %goto exit; 
    %end;
    %if ( &ng>1 &(^&notestflag)) %then %do;
      Title "Gray's Test for Equality of Cumulative Incidence Functions";
      proc print data=testresult NOOBS Label;
        label test_stat="Chi-Square"
        df="DF"
        pval= "Pr > Chi-Square";
        format pval pvalue6.4;
      run;
    %end;

    %exit:
    Title ;
    Title2 ;
    proc datasets nolist;
      delete &data.init outgrplabel _c;
    run;

    options &savenote;

  %mend;

