/* Macro library for creating survival analysis tables
 * for lifetime data with or without competing risks
 * Maintainer: Zhen-Huan Hu <zhu@mcw.edu>
 * Last change: Dec 5, 2014

 * Functions and their uses:
 *
 * %lifetest(indata =, event =, competerisk =, intv =, strata =, subgroup =, timelist =);
 * %lifetestexport(outdata =, rtftitle =);
 *
 * INDATA:	  Input dataset
 * EVENT:	  Event of interest (default: dead)
 * COMPETERISK:   Competing risk to the event of interest
 * INTV:	  Time to the event / censoring
 * STRATA:	  Main comparing group
 * SUBGROUP:	  Secondary comparing group
 * TIMELIST:	  Specific time points for survival estimates
 * OUTDATA:	  Output file name
 * RTFTITLE:	  Title for the output table
 */

%let ttlcnt = 0; %* Total number of tables;
%let colcnt = 0; %* Total number of columns;

%macro ltrelease(); %* Clean the table stack;
  title;
  %let ttlcnt = 0;
  %let colcnt = 0;
%mend;

%macro multiztest(indata =, mean =, std =, group = nil);
  data _multiztest_cache_;
    set &indata;
    keep &mean &std &group;
    where not missing(&mean) and not missing(&std) 
    %if &group ~= nil %then %do; and not missing(&group) %end;;
    %if &group = nil %then %do; nil = 1; %end;
  run;

  proc sql noprint;
    select count(distinct &group) into :ngrp trimmed
    from _multiztest_cache_;

    select count(&group), mean(&group) as grpvalue
    into :nsample1 - :nsample&ngrp, :grpvalue1 - :grpvalue&ngrp
    from _multiztest_cache_ group by &group order by grpvalue;
    
    select min(samplesize) into :nsamplemin trimmed
    from (select count(&group) as samplesize 
    from _multiztest_cache_ group by &group);
  quit;

  %do t = 1 %to &ngrp;
    %if &&nsample&t <= 1 %then %do;
      data pvalue&t;
	length chisq df 8;
	call missing(chisq, df);
      run;
      %put WARNING: Need more than 1 sample to perform test.;
    %end;
    %else %do;
      proc iml;
	use _multiztest_cache_ where(&group = &&grpvalue&t);
	read all var {&mean &std};
	nsample = nrow(&mean); %* Number of samples;
	p1 = &mean[1 : (nsample - 1)];
	p2 = &mean[2 : nsample];
	x = p1 - p2;
	var = &std ## 2;
	sigma = j(nsample - 1, nsample - 1, 0); %* Init variance matrix;
	do i = 1 to (nsample - 1);
	  sigma[i, i] = var[i] + var[i + 1];
	end;
	do i = 1 to (nsample - 2);
	  sigma[i, i + 1] = -var[i + 1];
	  sigma[i + 1, i] = sigma[i, i + 1];
	end;
	chisq = t(x) * inv(sigma) * x;
	out = j(1, 2);
	out[1] = chisq; out[2] = nsample - 1;
	create pvalue&t from out[colname = {'chisq' 'df'}];
	append from out;
	close pvalue&t;
      quit;
    %end;
  %end;

  data _pointwise_pvalue_ (keep = &group p);
    set %do t = 1 %to &ngrp; pvalue&t (in = in&t) %end;;
    if not missing(chisq) and not missing(df) then p = 1 - probchi(chisq, df);
    %do t = 1 %to &ngrp; if in&t then &group = &&grpvalue&t; %end;
  run;
%mend;

%macro cifstats2(indata =, outdata = , group = nil, event =, competerisk =, intv =, timelist = 12 36 60, conftype = 2);
  %* Initializing data;
  data _cif_init_;
    set &indata;
    where &event >= 0 and &competerisk >= 0 and &intv >= 0 and &group >= 0;

    %* Create dummy group if GROUP not specified;
    %if &group = nil %then %do;
      nil = 1;
      label nil = "CIF";
    %end;

    %* Create STATUS variable as required by the %CIF program;
    if &event = 1 then cifstatus = 1;
    else if &competerisk = 1 then cifstatus = 2;
    else if &event = 0 and &competerisk = 0 then cifstatus = 0;

    %* Keep relevant variables for the subsequent calculations;
    keep &group cifstatus &intv;
  run;

  %* Calculate cumulative incidence;
  %cif(data = _cif_init_, time = &intv, status = cifstatus, group = &group, out = &outdata, conftype = &conftype, options = nocifest noplot);

  %* Condensed timelist;
  %let ntimepoints = %sysfunc(countw(&timelist, %str( )));
  proc sort data = &outdata; by &group &intv;
  data &outdata (keep = timelist &intv &group cif cif_stderr cif_lcl cif_ucl left);
    set &outdata (rename = (cif = d_cif &intv = d_&intv)); by &group d_&intv;
    retain i (1) t1-t&ntimepoints (&timelist) &intv timelist cif cif_stderr cif_lcl cif_ucl left;
    array tlist {&ntimepoints} t1-t&ntimepoints;

    * Initialize;
    if first.&group then do;
      i = 1;
      &intv = d_&intv;
      if not missing(d_cif) then do;
        cif = d_cif;
        cif_stderr = stderr;
        cif_lcl = lowerci;
        cif_ucl = upperci;
        left = natrisk;
      end;
    end;

    * Select output observation;
    if i <= &ntimepoints then do while (d_&intv > tlist{i});
      timelist = tlist{i};
      output &outdata;
      i = i + 1;
      if i > &ntimepoints then leave;
    end;

    if i <= &ntimepoints then do; 
      &intv = d_&intv;
      if not missing(d_cif) then do;
        cif = d_cif;
        cif_stderr = stderr;
        cif_lcl = lowerci;
        cif_ucl = upperci;
        left = natrisk;
      end;
    end;

    * Make sure the last observation is checked;
    if last.&group = 1 and i <= &ntimepoints 
    and d_&intv <= tlist{&ntimepoints} then do while (i <= &ntimepoints);
      &intv = d_&intv;
      timelist = tlist{i};
      if not missing(d_cif) then do;
        cif = d_cif;
        cif_stderr = stderr;
        cif_lcl = lowerci;
        cif_ucl = upperci;
        left = natrisk;
      end;
      output &outdata;
      i = i + 1;
    end;
  run;
%mend;

%macro lifetestlite(indata = ,		      /* Input SAS dataset */
		    event = dead,	      /* Event: death, dfs, relapse, etc. */
		    competerisk = nil,	      /* Competing risk */
		    intv = intxsurv,	      /* Time to event */
		    timelist = 12 36 60,      /* Specific time points for estimation */
		    strata = nil,	      /* Main group */
		    pwpvalue = 0,	      /* Estimate point-wise p-values */
		    leftcheck = 1,	      /* Ignore the estimate if less than 15 obs left */
		    footnote = 6,	      /* Footnote if the last obs is 6 months earlier */
		    conftype = 2	      /* Type of transformation used when calculating confidence limits */
		    );

  %if &footnote < 0 %then %let footnote = 0;
  proc format; value nil 1 = "Study population";

  %* Count the # of obs by strata;
  data _cnt_init_;
    set &indata;
    %if &strata = nil %then %do; nil = 1; format nil nil.; %end;
    %else %do; where not missing(&strata); %end;
    keep &event &intv &strata %if &competerisk ~= nil %then %do; &competerisk %end;;
  run;

  proc sql noprint;
    %global colcnt;
    select count(distinct &strata) 
    into :colcnt trimmed from _cnt_init_;
    %do i = 1 %to &colcnt;
      %global sttcnt&i sttlab&i sttname&i;
    %end;
    select count(&strata) as count, mean(&strata) as value
    into :sttcnt1 - :sttcnt&colcnt, :sttvalue1 - :sttvalue&colcnt from _cnt_init_ 
    group by &strata order by value;
  quit;

  %* Preload dataset for analysis;
  data _lifetest_init_;
    set _cnt_init_;
    where &intv >= 0 and not missing(&event)
    %if &competerisk ~= nil %then %do; and not missing(&competerisk) %end;;
    call symput("eventlabel", vlabel(&event));
  run;

  /* Create header */
  proc freq data = _lifetest_init_ noprint;
    tables &strata / out = _strata_(drop = percent);
  proc sort data = _strata_; by &strata;
  proc transpose data = _strata_ out = _header_(drop = _name_ _label_);
    var count;
    id &strata;
    idlabel &strata;
  run;

  proc contents data = _header_ out = _vars_(keep = varnum name label) noprint;
  proc sort data = _vars_; by varnum;
  data _vars_;
    set _vars_;
    call symput(cat("sttname", strip(_n_)), name);
    label = prxchange("s/^-?\d+\.?\s+(.*)/\u$1/", 1, strip(label));
    if label ne "" then call symput(cats("sttlab", _n_), label);
    else call symput(cats("sttlab", _n_), name);
  run;

  /* Analysis */
  %local estimate lcl ucl stderr;
  %* Survivor function;
  %if &competerisk = nil %then %do;
    ods listing close;
    proc lifetest data = _lifetest_init_ timelist = &timelist 
      outsurv = _output_ stderr reduceout conftype = 
      %if &conftype = 1 %then %do; loglog %end;
      %else %if &conftype = 2 %then %do; asinsqrt %end;
      %else %if &conftype = 3 %then %do; logit %end;
      ;
      strata &strata;
      time &intv * &event(0);
      ods output ProductLimitEstimates = _number_left_(keep = &strata timelist left);
      ods output HomTests = testresult;
    run;
    ods listing;

    proc sort data = _output_; by &strata timelist;
    proc sort data = _number_left_; by &strata timelist;
    data _output_;
      merge _output_ _number_left_;
      by &strata timelist;
    run;

    %let estimate = survival;
    %let lcl = sdf_lcl;
    %let ucl = sdf_ucl;
    %let stderr = sdf_stderr;
  %end;
  %* Cumulative incidence function;
  %else %do;
    %cifstats2(indata = _lifetest_init_, outdata = _output_, group = &strata,
    event = &event, competerisk = &competerisk, intv = &intv, timelist = &timelist,
    conftype = &conftype);

    %let estimate = cif;
    %let lcl = cif_lcl;
    %let ucl = cif_ucl;
    %let stderr = cif_stderr;
  %end;

  %* Point-wise p-values;
  %if &pwpvalue = 1 %then %do;
    %multiztest(indata = _output_, mean = &estimate, std = &stderr, group = timelist);
  %end;

  /* Section: post processing */

  %* We need to check the actual time of the last event in each time period;
  %* If it is 6 months away from the nearest time point, make a footnote;
  data _output_;
    set _output_;

    lastevent = timelist - &intv;
    &estimate = &estimate * 100;
    &lcl = &lcl * 100;
    &ucl = &ucl * 100;
    &stderr = &stderr * 100;

    length result $ 250;

    %if &leftcheck = 1 %then %do;
      if left < 15 then result = "NE";
    %end;
    %else %if &leftcheck = 0 %then %do;
      %* Report in the log;
      if left < 15 then put "WARNING: only " left " obs left after "
      timelist " months. (event = &event, strata = &strata)";
      %* When survival drops to 0;
      if missing(&estimate) or missing(&lcl) or missing(&ucl) then result = "NE";
    %end;
    else if &estimate = 100 then result = "100%";
    %if &footnote > 0 %then %do;
      else if timelist > &footnote and lastevent > &footnote then
      result = cat(round(&estimate), " (", round(&lcl), "-", round(&ucl),
      ")%\R'{\super\chftn}{\footnote\pard\plain\fs20 {\super\chftn} Actual last event occurred in ", round(&intv), " months.}'");
    %end;
    else result = cat(round(&estimate), " (", round(&lcl), "-", round(&ucl), ")%");
  run;

  proc sort data = _output_; by timelist;
  proc transpose data = _output_ out = _t_output_(drop = _name_);
    var result;
    by timelist;
    id &strata;
    idlabel &strata;
  run;

  %if &pwpvalue = 1 %then %do;
    proc sort data = _t_output_; by timelist;
    proc sort data = _pointwise_pvalue_; by timelist;
    data _t_output_;
      merge _t_output_ _pointwise_pvalue_; by timelist;
    run;
  %end;

  data _header_;
    set _header_;
    timelist = -1; %* Dummy index used when merging;
    %do i = 1 %to &colcnt;
      length count&i $ 20;
      count&i = strip(put(&&sttname&i, 8.));
      label count&i = "N Eval";
      drop &&sttname&i;
    %end;
  run;

  %if &colcnt > 1 %then %do;
    %if &competerisk = nil %then %do;
      data _overall_pvalue_;
	set testresult;
	where test = "Log-Rank";
	rename probchisq = p;
	length pvalue $ 25;
	timelist = -1;
	keep timelist probchisq;
      run; 
    %end;
    %else %do;
      data _overall_pvalue_;
	set testresult;
	rename pval = p;
	timelist = -1;
	keep timelist pval;
      run;
    %end;

    proc sql noprint;
      create table _header_complete_ as
      select a.*, b.p from _header_ as a left join _overall_pvalue_ as b
      on a.timelist = b.timelist
      order by timelist;
    quit;
  %end;
  %else %if &colcnt = 1 %then %do;
    proc sql noprint;
      create table _header_complete_ as
      select * from _header_ 
      order by timelist;
    quit;
  %end;

  /* Merge header */
  %let ttlcnt = %eval(&ttlcnt + 1);
  data tab&ttlcnt;
    set _header_complete_ _t_output_;

    length varstr $ 100;
    if timelist = -1 then varstr = strip("&eventlabel");
    else if timelist = 0.921 then varstr = "\R'\tab' 28-day";
    else if 3.27 <= timelist <= 3.29 then varstr = "\R'\tab' 100-day";
    else if timelist = 3.75 then varstr = "\R'\tab' 114-day";
    else if timelist in (12, 24, 36, 48, 60) then varstr = cat("\R'\tab' ", round(timelist / 12), "-year");
    else varstr = cat("\R'\tab' ", round(timelist), " months");

    length pvalue $ 25;
    if p >= 0.01 then pvalue = strip(put(p, 4.2));
    else if p >= 0.001 then pvalue = strip(put(p, 5.3));
    else if p >= 0 then pvalue = "<0.001";

    drop timelist p;
  run;
%mend;

%macro lifetestpro(indata =,		/* Input SAS dataset */
		  event = dead,		/* Evant: dead, dfs, relapse, etc. */
		  intv = intxsurv,	/* Time to event */
		  competerisk = nil,	/* Competing risk */
		  strata = nil,		/* Main group */
		  subgroup =,		/* Secondary group */
		  timelist = 12 36 60,	/* Specific time points for estimation */
		  pwpvalue = 0,		/* Point-wise p-value */
		  leftcheck = 1,	/* Ignore the estimate if less than 15 obs left */
		  footnote = 6,		/* Footnote if the last obs is 6 months earlier */
		  conftype = 2		/* Type of transformation used when calculating confidence limits */
		  );

  %if &footnote < 0 %then %let footnote = 0;
  proc format; value nil 1 = "Study population";

  %* Count the # of obs by strata;
  data _cnt_init_;
    set &indata;
    %if &strata = nil %then %do; nil = 1; %end;
    %else %do; where not missing(&strata); %end;
    keep &strata;
  run;

  proc sql noprint;
    %global colcnt;
    select count(distinct &strata) 
    into :colcnt trimmed from _cnt_init_;
    %do i = 1 %to &colcnt;
      %global sttcnt&i sttlab&i sttname&i;
    %end;
    select count(&strata) as count, mean(&strata) as value
    into :sttcnt1 - :sttcnt&colcnt, :sttvalue1 - :sttvalue&colcnt from _cnt_init_ 
    group by &strata order by value;
  quit;

  %* Preload dataset for analysis;
  data _init_;
    %if &strata = nil %then %do;
      %if &competerisk = nil %then %do;
        set &indata (keep = &event &intv &subgroup);
        where &intv >= 0 and not missing(&subgroup) and not missing(&event);
      %end;
      %else %do;
        set &indata (keep = &event &intv &subgroup &competerisk);
        where &intv >= 0 and not missing(&subgroup) and not missing(&event)
	and not missing(&competerisk);
      %end;
      nil = 1; format nil nil.;
    %end;
    %else %do;
      %if &competerisk = nil %then %do;
        set &indata (keep = &event &intv &subgroup &strata);
        where &intv >= 0 and not missing(&subgroup) and not missing(&event)
	and not missing(&strata);
      %end;
      %else %do;
        set &indata (keep = &event &intv &subgroup &strata &competerisk);
        where &intv >= 0 and not missing(&subgroup) and not missing(&event)
	and not missing(&strata) and not missing(&competerisk);
      %end;
    %end;
    call symput("eventlabel", vlabel(&event));
    call symput("colfmt", vformat(&strata)); 
  run;

  /* Pull column names */
  proc freq data = _init_ noprint;
    tables &strata / out = _overall_strata_(drop = percent);
  proc sort data = _overall_strata_; by &strata;
  proc transpose data = _overall_strata_ out = _overall_header_(drop = _name_ _label_);
    var count;
    id &strata;
    idlabel &strata;
  run;

  proc contents data = _overall_header_ out = _overall_vars_(keep = varnum name label) noprint;
  proc sort data = _overall_vars_; by varnum;
  %do i = 1 %to &colcnt;
    %global sttname&i sttlab&i;
  %end;

  data _overall_vars_;
    set _overall_vars_;
    call symputx(cats("sttname", _n_), name); * Store column name;
    label = prxchange("s/^-?\d+\.?\s+(.*)/\u$1/", 1, strip(label));
    if label ne "" then call symputx(cats("sttlab", _n_), label); * Store column label;
    else call symputx(cats("sttlab", _n_), name);
  run;

  /* Construct subgroups */
  %* Total # of subgroups;
  proc sql noprint;
    select count(distinct &subgroup)
    into :subgrpcnt trimmed from _init_;
  quit;

  %* Pull subgroup variable value;
  proc sql noprint;
    select mean(&subgroup)
    into :subgrpvalue1 - :subgrpvalue&subgrpcnt from _init_
    group by &subgroup order by &subgroup;
  quit;

  proc freq data = _init_ noprint;
    table &subgroup / out = _subgrp_(drop = percent);
  proc sort data = _subgrp_; by &subgroup;
  proc transpose data = _subgrp_ out = _t_subgrp_(drop = _name_ _label_);
    var count;
    id &subgroup;
    idlabel &subgroup;
  run;

  proc contents data = _t_subgrp_ out = _subgrp_vars_(keep = varnum name label) noprint;
  proc sort data = _subgrp_vars_; by varnum;

  /* Table for each subgrp */
  %do s = 1 %to &subgrpcnt;
    %local subgrpvalue&s subgrpname&s subgrplabel&s;
    
    data _subgrp_var_;
      set _subgrp_vars_;
      where varnum = &s;
      call symputx("subgrpname&s", name); %* Pull subgroup name;
      if label ne "" then call symputx("subgrplabel&s", label); %* Pull subgroup label;
      else call symputx("subgrplabel&s", name);
    run;

    /* Analysis */
    %* Keep cases within the subgrp;
    data _lifetest_subgrp_init_;
      set _init_;
      where &subgroup = &&subgrpvalue&s;
    run;

    %local estimate lcl ucl stderr;
    %if &competerisk = nil %then %do;
      %* Survivor function;
      ods listing close;
      proc lifetest data = _lifetest_subgrp_init_ timelist = &timelist 
	outsurv = _output_ stderr reduceout conftype = 
	%if &conftype = 1 %then %do; loglog %end;
	%else %if &conftype = 2 %then %do; asinsqrt %end;
	%else %if &conftype = 3 %then %do; logit %end;
	;
	strata &strata;
	time &intv * &event(0);
	ods output ProductLimitEstimates = _number_left_(keep = &strata timelist left);
        ods output HomTests = testresult;
      run;
      ods listing;

      proc sort data = _output_; by &strata timelist;
      proc sort data = _number_left_; by &strata timelist;
      data _output_;
	merge _output_ _number_left_;
	by &strata timelist;
      run;

      %let estimate = survival;
      %let lcl = sdf_lcl;
      %let ucl = sdf_ucl;
      %let stderr = sdf_stderr;
    %end;
    %* Cumulative incidence function;
    %else %do;
      %cifstats2(indata = _lifetest_subgrp_init_, outdata = _output_, group = &strata, 
      event = &event, competerisk = &competerisk, intv = &intv, timelist = &timelist,
      conftype = &conftype);

      %let estimate = cif;
      %let lcl = cif_lcl;
      %let ucl = cif_ucl;
      %let stderr = cif_stderr;
    %end;

    %* Point-wise p-values;
    %if &pwpvalue = 1 %then %do;
      %multiztest(indata = _output_, mean = &estimate, std = &stderr, group = timelist);
    %end;

    /* Post processing */
    %* We need to check the actual time of the last event in each time period;
    %* If it is 6 months away from the nearest time point, make a footnote;
    data _output_;
      set _output_;

      lastevent = timelist - &intv;
      &estimate = &estimate * 100;
      &lcl = &lcl * 100;
      &ucl = &ucl * 100;
      &stderr = &stderr * 100;

      length result $ 250;

      %if &leftcheck = 1 %then %do;
	if left < 15 then result = "NE";
      %end;
      %else %if &leftcheck = 0 %then %do;
        %* Report in the log;
        if left < 15 then put "WARNING: only " left " obs left after " 
	timelist " months. (event = &event, strata = &strata)";
	%* When survival drops to 0;
	if missing(&estimate) or missing(&lcl) or missing(&ucl) then result = "NE";
      %end;
      else if &estimate = 100 then result = "100%";
      %if &footnote > 0 %then %do;
	else if timelist > &footnote and lastevent > &footnote then
       	result = cat(round(&estimate), " (", round(&lcl), "-", round(&ucl),
	")%\R'{\super\chftn}{\footnote\pard\plain\fs20 {\super\chftn} Actual last event occurred in ", round(&intv), " months.}'");
      %end;
      else result = cat(round(&estimate), " (", round(&lcl), "-", round(&ucl), ")%");
    run;
  
    proc sort data = _output_; by timelist;
    proc transpose data = _output_ out = _t_output_(drop = _name_);
      var result;
      by timelist;
      id &strata;
    run;

    %if &pwpvalue = 1 %then %do;
      proc sort data = _t_output_; by timelist;
      proc sort data = _pointwise_pvalue_; by timelist;
      data _t_output_;
	merge _t_output_ _pointwise_pvalue_; by timelist;
      run;
    %end;

    /* Create header */
    proc freq data = _init_ noprint;
      where &subgroup = &&subgrpvalue&s; * Only need cases for the subgrp;
      tables &strata / out = _strata_(drop = percent);
    proc sort data = _strata_; by &strata;
    proc transpose data = _strata_ out = _header_(drop = _name_ _label_);
      var count;
      id &strata;
      idlabel &strata;
    run;

    data _header_;
      set _header_;
      timelist = -1;
      %do i = 1 %to &colcnt;
	if missing(&&sttname&i) then &&sttname&i = 0;
        length count&i $ 20;
	* Total # of obs in each column of the subgroup;
	if &&sttname&i > 0 then count&i = strip(put(&&sttname&i, 8.));
	else count&i = "";
        label count&i = "N Eval";
        drop &&sttname&i;
      %end;
    run;

    %if &colcnt > 1 %then %do;
      %if &competerisk = nil %then %do;
	data _overall_pvalue_;
	  set testresult;
	  where test = "Log-Rank";
	  rename probchisq = p;
	  length pvalue $ 25;
	  timelist = -1;
	  keep timelist probchisq;
	run; 
      %end;
      %else %do;
	data _overall_pvalue_;
	  set testresult;
	  rename pval = p;
	  timelist = -1;
	  keep timelist pval;
	run;
      %end;

      proc sql noprint;
	create table _header_complete_ as
	select a.*, b.p from _header_ as a left join _overall_pvalue_ as b
	on a.timelist = b.timelist
	order by timelist;
      quit;
    %end;
    %else %if &colcnt = 1 %then %do;
      proc sql noprint;
	create table _header_complete_ as
	select * from _header_ 
	order by timelist;
      quit;
    %end;

    /* Merge header */
    data _tab_&s;
      set _header_complete_ _t_output_;

      length varstr $ 100;
      if timelist = -1 then varstr = cat("\R'\tab' ", prxchange("s/^-?\d+\.?\s+(.*)/\u$1/", 1, strip("&&subgrplabel&s"))); 
      else if timelist = 0.921 then varstr = "\R'\tab\tab' 28-day";
      else if 3.27 <= timelist <= 3.29 then varstr = "\R'\tab\tab' 100-day";
      else if timelist = 3.75 then varstr = "\R'\tab\tab' 114-day";
      else if timelist in (12, 24, 36, 48, 60) then varstr = cat("\R'\tab\tab' ", round(timelist / 12), "-year");
      else if not missing(timelist) then varstr = cat("\R'\tab\tab' ", round(timelist), " months");
  
      length pvalue $ 25;
      if p >= 0.01 then pvalue = strip(put(p, 4.2));
      else if p >= 0.001 then pvalue = strip(put(p, 5.3));
      else if p >= 0 then pvalue = "<0.001";

      drop timelist p;
    run;
  %end;

  data _main_header_;
    length varstr $ 100;
    varstr = strip("&eventlabel");
  run;

  %let ttlcnt = %eval(&ttlcnt + 1);
  data tab&ttlcnt;
    set _main_header_ %do s = 1 %to &subgrpcnt; _tab_&s %end;;
  run;
%mend;

%macro lifetest(indata =,		/* Input SAS dataset */
		event = dead,		/* Evant: dead, dfs, relapse, etc. */
		intv = intxsurv,	/* Time to event */
	        competerisk = nil,	/* Competing risk */
		strata = nil,		/* Main group */
		subgroup = nil,		/* Secondary group */
		timelist = 12 36 60,	/* Time points of the estimates */
		pwpvalue = 0,		/* Estimate point-wise p-values */
		leftcheck = 1,		/* Ignore the estimates if <= 15 obs left */
		footnote = 6		/* Footnote if the last obs is 6 months earlier */
		);

  %* Precheck all variables are ready;
  %if &indata eq %then %do;
    %put ERROR: Please specify the input data set;
    %goto exit;
  %end;
  %else %if not %sysfunc(exist(&indata)) %then %do;
    %put ERROR: Cannot find &indata;
    %goto exit;
  %end;

  %if &subgroup = nil %then %do;
    %lifetestlite(indata = &indata, competerisk = &competerisk,
    event = &event, intv = &intv, strata = &strata, timelist = &timelist, 
    pwpvalue = &pwpvalue, leftcheck = &leftcheck, footnote = &footnote);
  %end;
  %else %do;
    %lifetestpro(indata = &indata, competerisk = &competerisk,
    event = &event, intv = &intv, strata = &strata, subgroup = &subgroup, timelist = &timelist,
    pwpvalue = &pwpvalue, leftcheck = &leftcheck, footnote = &footnote);
  %end;

  %exit:
%mend lifetest;

%macro lifetestexport(outdata = lifetest, /* Output SAS dataset */
		      rtftitle =,	  /* Title of the RTF file */
		      pvalue = 1,	  /* Show p-value in final report */
		      style = custom	  /* Output RTF style */
		      );

  %if &ttlcnt = 0 %then %do;
    %put ERROR: Life test has not been initialized.;
    %goto exit;
  %end;

  %if &colcnt = 1 %then %let pvalue = 0;
  %do i = 1 %to &colcnt;
    %let sttlab&i = %sysfunc(strip(&&sttlab&i));
    %let sttcnt&i = %sysfunc(strip(&&sttcnt&i));
  %end;

  %let hcolsize = 45; %* 1st column size;
  %let ncolsize = 10; %* N column size;
  %let ecolsize = 20; %* Estimates column size;

  %* p-value column size;
  %if &pvalue = 1 %then %let pcolsize = 10;
  %else %if &pvalue = 0 %then %let pcolsize = 0;

  %* Dynamic line size;
  %let linesize = %eval(4 + &hcolsize + &colcnt * (&ncolsize + 
    &ecolsize + 4) + &pcolsize);
  %if &linesize > 256 %then %let linesize = max;
  %else %if &linesize < 64 %then %let linesize = 64;

  options ls = &linesize nodate nocenter;
  ods rtf file = "&outdata..rtf" style = &style bodytitle;
  ods escapechar = '\';

  data _lifetest_final_;
    set %do i = 1 %to &ttlcnt; tab&i %end;;
  run;

  title justify = left &rtftitle;
  proc report data = _lifetest_final_ nowd split = '|';
    column varstr %do i = 1 %to &colcnt;
    ("&&sttlab&i (N = &&sttcnt&i)" count&i &&sttname&i) %end;
    %if &pvalue = 1 %then %do; pvalue %end;;
    define varstr / "Outcomes" order = data style = [just = left] width = &hcolsize;
    %do i = 1 %to &colcnt;
      define count&i / "N Eval" order = data style = [just = right] width = &ncolsize;
      define &&sttname&i / "Prob (95% CI)" order = data style = [just = right] width = &ecolsize;
    %end;
    %if &pvalue = 1 %then %do;
      define pvalue / "p-value" order = data style = [just = right] width = &pcolsize;
    %end;
  run;

  ods rtf close;
  %ltrelease(); %* Reset;
  %exit:
%mend lifetestexport;
